import createDecorator from '@loafer/core/util/decorators/createDecorator';
import InjectableDefinition from '@loafer/context/definition/InjectableDefinition';
import { getInjectableDefinition } from '@loafer/context/util/metadata';

import GetAppContext from '@loafer/context';
import { validateQualifier } from '@loafer/pouches/util/qualifier';
import DefaultPouchFactory from '@loafer/pouches/factory/implement/DefaultPouchFactory';
import PouchDefinitionRegistry from '@loafer/pouches/factory/registry/PouchDefinitionRegistry';

const Injectable = (qualifier?: string | symbol ) => createDecorator('Injectable', {
  classDecorator: <TFunction extends Function>(target: TFunction): TFunction | void => {
    let injectableDefinition: InjectableDefinition = getInjectableDefinition(target.prototype, false);
    if (undefined !== injectableDefinition) {
      throw new Error('Cannot apply @Injectable decorator multiple times.');
    }

    injectableDefinition = getInjectableDefinition(target.prototype, true);
    injectableDefinition.Qualifier = validateQualifier(target.prototype, qualifier);

    let pouchFactory: DefaultPouchFactory = GetAppContext().getPouchFactory();
    let pouchDefinitionRegistry = <PouchDefinitionRegistry>pouchFactory;
    pouchDefinitionRegistry.registerPouchDefinition(injectableDefinition);

    return target;
  },

});


export default Injectable;
